#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)


import kaldifst


def test_float_weight():
    w = kaldifst.FloatWeight(2.0)
    assert w.value == 2.0
    assert w == kaldifst.FloatWeight(2.0)
    assert str(w) == "2"
    # Note: You cannot change w.value once it is set


def test_tropical_weight():
    w = kaldifst.TropicalWeight(1)
    assert w.value == 1

    # Note: You cannot change w.value once it is set
    w2 = kaldifst.TropicalWeight.zero
    assert w2 == float("inf")

    w3 = kaldifst.TropicalWeight.one
    assert w3 == 0.0

    w4 = kaldifst.TropicalWeight.no_weight
    assert str(w4) == "nan"


def test_tropical_weight_operations():
    w1 = kaldifst.TropicalWeight(2)
    w2 = kaldifst.TropicalWeight(3)

    w3 = kaldifst.plus(w1, w2)  # return the minimum, min(w1, w2)
    assert w3 == 2

    w4 = kaldifst.times(w1, w2)  # return the sum: w1 + w2
    assert w4 == 5

    w5 = kaldifst.divide(w1, w2)  # return thed difference: w1 - w2
    assert w5 == -1


def main():
    test_float_weight()
    test_tropical_weight()
    test_tropical_weight_operations()


if __name__ == "__main__":
    main()



# Copyright      2021  Xiaomi Corp.        (authors: Fangjun Kuang)
# See ../../../LICENSE for clarification regarding multiple authors

from _kaldifst import (
    StdConstFst,
    StdFst,
    StdVectorFst,
    _ArcIteratorStdConstFst,
    _ArcIteratorStdVectorFst,
    _StateIteratorStdConstFst,
    _StateIteratorStdVectorFst,
)


class StateIterator(object):
    def __init__(self, fst: StdFst):
        if isinstance(fst, StdVectorFst):
            self._impl = _StateIteratorStdVectorFst(fst)
        elif isinstance(fst, StdConstFst):
            self._impl = _StateIteratorStdConstFst(fst)
        else:
            raise f"Unsupported type: {type(fst)}"

    def __iter__(self):
        while not self._impl.done:
            yield self._impl.value
            self._impl.next()

    @property
    def done(self):
        return self._impl.done

    @property
    def value(self):
        return self._impl.value

    def next(self):
        self._impl.next()

    def reset(self):
        self._impl.reset()


class ArcIterator(object):
    def __init__(self, fst: StdFst, state: int):
        if isinstance(fst, StdVectorFst):
            self._impl = _ArcIteratorStdVectorFst(fst, state)
        elif isinstance(fst, StdConstFst):
            self._impl = _ArcIteratorStdConstFst(fst, state)
        else:
            raise ValueError(f"Unsupported type: {type(fst)}")

    def __iter__(self):
        while not self._impl.done:
            yield self._impl.value
            self._impl.next()

    @property
    def done(self):
        return self._impl.done

    @property
    def value(self):
        return self._impl.value

    @property
    def position(self):
        return self._impl.position

    @property
    def flags(self):
        return self._impl.flags

    def next(self):
        return self._impl.next()

    def reset(self):
        return self._impl.reset()

    def seek(self, a: int):
        return self._impl.seek(a)



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)


import kaldifst


def test_arc_sort():
    s = """
    0 1 1 4
    0 1 3 5
    0 1 2 3
    0 2 5 2
    0 2 4 1
    1 2 2 3
    1 2 3 1
    1 2 1 2
    2
    """
    fst = kaldifst.compile(s, acceptor=False)

    assert fst.is_ilabel_sorted is False
    assert fst.is_olabel_sorted is False

    s1 = kaldifst.draw(fst, acceptor=False, portrait=True)
    print(s1)  # see https://git.io/JSVap

    kaldifst.arcsort(fst, sort_type="ilabel")

    assert fst.is_ilabel_sorted is True
    assert fst.is_olabel_sorted is False

    s2 = kaldifst.draw(fst, acceptor=False, portrait=True)
    print(s2)  # see https://git.io/JSVwK

    kaldifst.arcsort(fst, sort_type="olabel")

    assert fst.is_ilabel_sorted is False
    assert fst.is_olabel_sorted is True

    s3 = kaldifst.draw(fst, acceptor=False, portrait=True)
    print(s3)  # see https://git.io/JSVr8

    # see also https://www.openfst.org/twiki/bin/view/FST/ArcSortDoc


def main():
    test_arc_sort()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)

import os

import kaldifst


def test_symbol_table():
    assert kaldifst.SymbolTable.kNoSymbol == -1
    sym = kaldifst.SymbolTable()
    sym.add_symbol("a", 1)
    sym.add_symbol("b", 2)
    c_id = sym.add_symbol("c")  # return the ID of "c"
    assert c_id == 3
    sym.name = "test"
    assert sym.name == "test"

    assert 1 in sym
    assert 2 in sym
    assert 3 in sym
    assert "a" in sym
    assert "b" in sym
    assert "c" in sym

    assert sym.find(1) == "a"
    assert sym.find(2) == "b"
    assert sym.find(3) == "c"
    assert sym.find(100) == ""  # return an empty string

    assert sym.find("a") == 1
    assert sym.find("b") == 2
    assert sym.find("c") == 3
    assert sym.find("d") == kaldifst.SymbolTable.kNoSymbol == -1

    assert sym.num_symbols() == 3
    assert sym.available_key() == 4  # return the highest key + 1

    sym.remove_symbol(2)  # can only remove an ID
    assert "b" not in sym

    assert sym.num_symbols() == 2
    assert sym.available_key() == 4  # return the highest key + 1

    sym.remove_symbol(3)
    assert sym.num_symbols() == 1

    hello_id = sym.add_symbol("hello")
    assert sym.find(hello_id) == "hello"
    assert sym.find("hello") == hello_id

    # write binary format
    sym.write("sym.bin")
    sym2 = kaldifst.SymbolTable.read("sym.bin")
    assert sym.check_sum == sym2.check_sum
    assert sym.labeled_check_sum == sym2.labeled_check_sum
    assert sym2.name == "test"

    # write text format
    sym.write_text("sym.txt")
    sym2 = kaldifst.SymbolTable.read_text("sym.txt")
    assert sym.check_sum == sym2.check_sum
    assert sym.labeled_check_sum == sym2.labeled_check_sum
    assert sym2.name == "sym.txt"

    os.remove("sym.bin")
    os.remove("sym.txt")


if __name__ == "__main__":
    test_symbol_table()
